#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
This program takes a short TDTS record of terahertz spectra of ca. 70 cm of air in the laboratory.
The ringing after the pump pulse corresponds to the water vapor absorption spectra. 

Both Fourier transform and FDM are used to separate the resonant peaks. While Fourier transform gives 
only a very rough spectrum, FDM detected these frequencies of oscillators [THz]: 
    0.62562     0.75036         1.073696                    1.409286 (...)

Experimental resonances from Exter M., et al.,: "Terahertz time-domain spectroscopy of water vapor":
    0.558       0.753  0.989    1.099   1.115 1.165 1.209   1.412

Conclusion: FDM can detect peaks with good accuracy even from very short experimental data.

Filip Dominec, dominecf@fzu.cz
"""


## Import common moduli
import numpy as np
from scipy.constants import c, hbar, pi
import matplotlib
matplotlib.rc('text', usetex=True)
matplotlib.rc('text.latex', preamble = \
        '\usepackage{amsmath}, \usepackage{yfonts}, \usepackage{txfonts}, \usepackage{lmodern},')
import matplotlib.pyplot as plt


## Getting 1D data
(x, y) = np.loadtxt("input.dat", usecols=list(range(2)), unpack=True)

## Load header to the 'parameters' dictionary
parameters = {}
with open('input.dat') as datafile:
    for line in datafile:
        if (line[0:1] in '0123456789') or ('column' in line.lower()): break    # end of parameter list
        key, value = line.replace(',', ' ').split()[-2:]
        try: value = float(value) ## Try to convert to float, if possible
        except: pass                ## otherwise keep as string
        parameters[key] = value



##Start figure + subplot (interactive)
fig = plt.figure(figsize=(10,10))
ax = plt.subplot(211, axisbg='#f8f8f8')
fig.subplots_adjust(left=.10, bottom=.08, right=.98, top=.98, wspace=-.05, hspace=.18)

# ========= Time domain ==========
## Plot line
plt.plot(x, np.real(y), color="#88FF00", label=u"$E(t)$ full record", ls='-', lw=3)
#plt.ylim((-0.1,1.1)); plt.xscale('linear')
#plt.xlim((-0.1,1.1)); plt.xscale('linear')
#Crop and decimate data for harminv
xr = np.linspace(2., max(x), 150)
yr = np.interp(xr, x, y)
plt.plot(xr, yr, color="#000000", label=u"$E(t)$ used for FFT and harminv", ls='-')      # (optional) plot amplitude
    
with open('hiinput.dat', 'w') as outfile: 
    outfile.write("#freq[THz]\t E(t)\n")
    np.savetxt(outfile, zip(xr, yr), fmt="%.8e")

## Finish subplot
plt.ylabel(u"Amplitude"); 
plt.xlabel(u"Time [ps]"); 
plt.grid()
plt.legend(prop={'size':10}, loc='upper right')

x = xr
y = yr

# ========= Fourier transform ==========
freq = np.fft.fftshift(np.fft.fftfreq(len(x), d=(x[1]-x[0])))         # calculate the frequency axis with proper spacing
yf = np.fft.fftshift(np.fft.fft(y)) / len(x) * 2*np.pi                 # calculate the FFT values, normalize to number of data points
truncated = np.logical_and(freq>0, freq<3)         # (optional) get the frequency range
(yf, freq) = map(lambda x: x[truncated], (yf, freq))    # (optional) truncate the data points

## Plot FT
ax = plt.subplot(212, axisbg='#f8f8f8')
plt.plot(freq, np.abs(yf), color="#FF8800", label=u"FFT", ls='-')      # (optional) plot amplitude
#plt.plot(freq, np.unwrap(np.angle(yf)), color="#FF8800", label=u"$y'$", ls='--')   # (optional) plot phase



# ========= harminv ==========
## Harminv
import subprocess
dt = x[1]-x[0]
subprocess.Popen('harminv 0-1000 -t %g -d 1 -f 103 < hiinput.dat > hiout.dat' % dt, shell=True, stdout=subprocess.PIPE).stdout.read().strip()

(mf, md, mQ, mA, mp, merr) = np.loadtxt('hiout.dat', usecols=list(range(6)), unpack=True, delimiter=', ', skiprows=1)
print mf*2, md
plt.errorbar(mf*2, mA*abs(mQ)/np.pi/2, yerr=merr, color="#0088FF", label=u"harminv", ls='', marker='*')      # (optional) plot amplitude

freq_fine = np.linspace(0, max(freq), 300)
def lorentz(x, f, d, q, A, p, err):
    return A*abs(q)/(1 + abs(q*np.pi*2)*abs(x-f*2)**2) / (np.pi*2)
sumosc = np.zeros_like(freq_fine)
for osc in np.vstack([mf, md, mQ, mA, mp, merr]).T:
    osc_y = lorentz(freq_fine, *osc)
    plt.plot(freq_fine, osc_y, color="#0088FF", label=u"", ls='-', alpha=.3)      # (optional) plot amplitude
    sumosc += osc_y 
plt.plot(freq_fine, sumosc, color="#0088FF", label=u"$\\Sigma$ osc", ls='-')      # (optional) plot amplitude

plt.xlim((-0.1,3.1)); plt.xscale('linear')
plt.ylabel(u"Amplitude"); 
plt.xlabel(u"Frequency [THz]"); 
plt.ylim((-0.1,1.5)); plt.yscale('linear')
#plt.xlim((-0.1,1.1)); plt.xscale('linear')
plt.grid()
plt.legend(prop={'size':10}, loc='upper right')




plt.savefig("output.png")
